@@ -280,7 +280,7 @@ formatter_fail(Config) ->
     ok = logger:set_handler_config(?MODULE,formatter,{?MODULE,bad_return}),
     logger:notice(?msg,?domain),
     try_match_file(Log,
-                   escape(Got3)++"FORMATTER ERROR: bad_return_value",
+                   escape(Got3)++"FORMATTER ERROR: bad return value",
                    5000),
 
     %% Check that handler is still alive and was never dead
@@ -1570,27 +1570,29 @@ h_proc_name() ->
 h_proc_name(Name) ->
     ?name_to_reg_name(logger_std_h,Name).
 
-file_delete(Log) ->
-   file:delete(Log).
-
 wait_for_process_up(T) ->
-    wait_for_process_up(h_proc_name(),T).
+    wait_for_process_up(?MODULE, h_proc_name(), T).
 
-wait_for_process_up(Name,T) ->
+wait_for_process_up(Name, RegName, T) ->
     N = (T div 500) + 1,
-    wait_for_process_up1(Name,N).
+    wait_for_process_up1(Name, RegName, N).
 
-wait_for_process_up1(_Name,0) ->
+wait_for_process_up1(_Name, _RegName, 0) ->
     error;
-wait_for_process_up1(Name,N) ->
+wait_for_process_up1(Name, RegName, N) ->
     timer:sleep(500),
-    case whereis(Name) of
+    case whereis(RegName) of
         Pid when is_pid(Pid) ->
-            %% ct:pal("Process ~p up (~p tries left)",[Name,N]),
-            {ok,Pid};
+            case logger:get_handler_config(Name) of
+                {ok,_} ->
+                    %% ct:pal("Process ~p up (~p tries left)",[Name,N]),
+                    {ok,Pid};
+                _ ->
+                    wait_for_process_up1(Name, RegName, N-1)
+            end;
         undefined ->
             %% ct:pal("Waiting for process ~p (~p tries left)",[Name,N]),
-            wait_for_process_up1(Name,N-1)
+            wait_for_process_up1(Name, RegName, N-1)
     end.
 
 filesync_rep_int() ->
@@ -1598,3 +1600,8 @@ filesync_rep_int() ->
         true  -> 5500;
         false -> ?FILESYNC_REPEAT_INTERVAL + 500
     end.
+
+
+file_delete(Log) ->
+   file:delete(Log).
+
